@@ -12,18 +12,180 @@ from html.parser import HTMLParser
 
 LOGGER = logging.getLogger(__name__)
 BULLET_WIN1252 = "\x95"  # BULLET character in Windows-1252 encoding
+DEFAULT_HEADING_SIZES = dict(h1=24, h2=18, h3=14, h4=12, h5=10, h6=8)
+
+COLOR_DICT = {
+    "black": "#000000",
+    "navy": "#000080",
+    "darkblue": "#00008b",
+    "mediumblue": "#0000cd",
+    "blue": "#0000ff",
+    "darkgreen": "#006400",
+    "green": "#008000",
+    "teal": "#008080",
+    "darkcyan": "#008b8b",
+    "deepskyblue": "#00bfff",
+    "darkturquoise": "#00ced1",
+    "mediumspringgreen": "#00fa9a",
+    "lime": "#00ff00",
+    "springgreen": "#00ff7f",
+    "aqua": "#00ffff",
+    "cyan": "#00ffff",
+    "midnightblue": "#191970",
+    "dodgerblue": "#1e90ff",
+    "lightseagreen": "#20b2aa",
+    "forestgreen": "#228b22",
+    "seagreen": "#2e8b57",
+    "darkslategray": "#2f4f4f",
+    "darkslategrey": "#2f4f4f",
+    "limegreen": "#32cd32",
+    "mediumseagreen": "#3cb371",
+    "turquoise": "#40e0d0",
+    "royalblue": "#4169e1",
+    "steelblue": "#4682b4",
+    "darkslateblue": "#483d8b",
+    "mediumturquoise": "#48d1cc",
+    "indigo": "#4b0082",
+    "darkolivegreen": "#556b2f",
+    "cadetblue": "#5f9ea0",
+    "cornflowerblue": "#6495ed",
+    "rebeccapurple": "#663399",
+    "mediumaquamarine": "#66cdaa",
+    "dimgray": "#696969",
+    "dimgrey": "#696969",
+    "slateblue": "#6a5acd",
+    "olivedrab": "#6b8e23",
+    "slategray": "#708090",
+    "slategrey": "#708090",
+    "lightslategray": "#778899",
+    "lightslategrey": "#778899",
+    "mediumslateblue": "#7b68ee",
+    "lawngreen": "#7cfc00",
+    "chartreuse": "#7fff00",
+    "aquamarine": "#7fffd4",
+    "maroon": "#800000",
+    "purple": "#800080",
+    "olive": "#808000",
+    "gray": "#808080",
+    "grey": "#808080",
+    "skyblue": "#87ceeb",
+    "lightskyblue": "#87cefa",
+    "blueviolet": "#8a2be2",
+    "darkred": "#8b0000",
+    "darkmagenta": "#8b008b",
+    "saddlebrown": "#8b4513",
+    "darkseagreen": "#8fbc8f",
+    "lightgreen": "#90ee90",
+    "mediumpurple": "#9370db",
+    "darkviolet": "#9400d3",
+    "palegreen": "#98fb98",
+    "darkorchid": "#9932cc",
+    "yellowgreen": "#9acd32",
+    "sienna": "#a0522d",
+    "brown": "#a52a2a",
+    "darkgray": "#a9a9a9",
+    "darkgrey": "#a9a9a9",
+    "lightblue": "#add8e6",
+    "greenyellow": "#adff2f",
+    "paleturquoise": "#afeeee",
+    "lightsteelblue": "#b0c4de",
+    "powderblue": "#b0e0e6",
+    "firebrick": "#b22222",
+    "darkgoldenrod": "#b8860b",
+    "mediumorchid": "#ba55d3",
+    "rosybrown": "#bc8f8f",
+    "darkkhaki": "#bdb76b",
+    "silver": "#c0c0c0",
+    "mediumvioletred": "#c71585",
+    "indianred": "#cd5c5c",
+    "peru": "#cd853f",
+    "chocolate": "#d2691e",
+    "tan": "#d2b48c",
+    "lightgray": "#d3d3d3",
+    "lightgrey": "#d3d3d3",
+    "thistle": "#d8bfd8",
+    "orchid": "#da70d6",
+    "goldenrod": "#daa520",
+    "palevioletred": "#db7093",
+    "crimson": "#dc143c",
+    "gainsboro": "#dcdcdc",
+    "plum": "#dda0dd",
+    "burlywood": "#deb887",
+    "lightcyan": "#e0ffff",
+    "lavender": "#e6e6fa",
+    "darksalmon": "#e9967a",
+    "violet": "#ee82ee",
+    "palegoldenrod": "#eee8aa",
+    "lightcoral": "#f08080",
+    "khaki": "#f0e68c",
+    "aliceblue": "#f0f8ff",
+    "honeydew": "#f0fff0",
+    "azure": "#f0ffff",
+    "sandybrown": "#f4a460",
+    "wheat": "#f5deb3",
+    "beige": "#f5f5dc",
+    "whitesmoke": "#f5f5f5",
+    "mintcream": "#f5fffa",
+    "ghostwhite": "#f8f8ff",
+    "salmon": "#fa8072",
+    "antiquewhite": "#faebd7",
+    "linen": "#faf0e6",
+    "lightgoldenrodyellow": "#fafad2",
+    "oldlace": "#fdf5e6",
+    "red": "#ff0000",
+    "fuchsia": "#ff00ff",
+    "magenta": "#ff00ff",
+    "deeppink": "#ff1493",
+    "orangered": "#ff4500",
+    "tomato": "#ff6347",
+    "hotpink": "#ff69b4",
+    "coral": "#ff7f50",
+    "darkorange": "#ff8c00",
+    "lightsalmon": "#ffa07a",
+    "orange": "#ffa500",
+    "lightpink": "#ffb6c1",
+    "pink": "#ffc0cb",
+    "gold": "#ffd700",
+    "peachpuff": "#ffdab9",
+    "navajowhite": "#ffdead",
+    "moccasin": "#ffe4b5",
+    "bisque": "#ffe4c4",
+    "mistyrose": "#ffe4e1",
+    "blanchedalmond": "#ffebcd",
+    "papayawhip": "#ffefd5",
+    "lavenderblush": "#fff0f5",
+    "seashell": "#fff5ee",
+    "cornsilk": "#fff8dc",
+    "lemonchiffon": "#fffacd",
+    "floralwhite": "#fffaf0",
+    "snow": "#fffafa",
+    "yellow": "#ffff00",
+    "lightyellow": "#ffffe0",
+    "ivory": "#fffff0",
+    "white": "#ffffff",
+}
 
 
 def px2mm(px):
     return int(px) * 25.4 / 72
 
 
-def hex2dec(color="#000000"):
+def color_as_decimal(color="#000000"):
     if not color:
         return None
-    r = int(color[1:3], 16)
-    g = int(color[3:5], 16)
-    b = int(color[5:7], 16)
+
+    # Checks if color is a name and gets the hex value
+    hexcolor = COLOR_DICT.get(color.lower(), color)
+
+    if len(hexcolor) == 4:
+        r = int(hexcolor[1] * 2, 16)
+        g = int(hexcolor[2] * 2, 16)
+        b = int(hexcolor[3] * 2, 16)
+        return r, g, b
+
+    r = int(hexcolor[1:3], 16)
+    g = int(hexcolor[3:5], 16)
+    b = int(hexcolor[5:7], 16)
     return r, g, b
 
 
@@ -37,6 +199,7 @@ class HTML2FPDF(HTMLParser):
         li_tag_indent=5,
         table_line_separators=False,
         ul_bullet_char=BULLET_WIN1252,
+        heading_sizes=None,
         **_,
     ):
         """
@@ -78,7 +241,9 @@ class HTML2FPDF(HTMLParser):
         self.theader_out = self.tfooter_out = False
         self.table_row_height = 0
         self.heading_level = None
-        self.hsize = dict(h1=2, h2=1.5, h3=1.17, h4=1, h5=0.83, h6=0.67)
+        self.heading_sizes = dict(**DEFAULT_HEADING_SIZES)
+        if heading_sizes:
+            self.heading_sizes.update(heading_sizes)
         self._only_imgs_in_td = False
 
     def width2unit(self, length):
@@ -114,7 +279,7 @@ class HTML2FPDF(HTMLParser):
             else:
                 if self.heading_level:
                     self.pdf.start_section(data, self.heading_level - 1)
-                LOGGER.debug("write '%s'", data.replace("\n", "\\n"))
+                LOGGER.debug("write '%s' h=%d", data.replace("\n", "\\n"), self.h)
                 self.pdf.write(self.h, data)
 
     def _insert_td(self, data=""):
@@ -133,7 +298,7 @@ class HTML2FPDF(HTMLParser):
         else:
             align = self.td.get("align", "L")[0].upper()
             border = border and "LR"
-        bgcolor = hex2dec(self.td.get("bgcolor", self.tr.get("bgcolor", "")))
+        bgcolor = color_as_decimal(self.td.get("bgcolor", self.tr.get("bgcolor", "")))
         # parsing table header/footer (drawn later):
         if self.thead is not None:
             self.theader.append(((width, height, data, border, 0, align), bgcolor))
@@ -244,12 +409,13 @@ class HTML2FPDF(HTMLParser):
             self.pdf.ln(self.h)
             if attrs:
                 self.align = attrs.get("align")
-        if tag in self.hsize:
+        if tag in self.heading_sizes:
+            self.font_stack.append((self.font_face, self.font_size, self.font_color))
             self.heading_level = int(tag[1:])
-            k = self.hsize[tag]
-            self.pdf.ln(5 * k)
+            hsize = self.heading_sizes[tag]
             self.pdf.set_text_color(150, 0, 0)
-            self.pdf.set_font_size(12 * k)
+            self.set_font(size=hsize)
+            self.pdf.ln(self.h)
             if attrs:
                 self.align = attrs.get("align")
         if tag == "hr":
@@ -276,12 +442,12 @@ class HTML2FPDF(HTMLParser):
                 self.bullet[self.indent - 1] = bullet
                 bullet = f"{bullet}. "
             self.pdf.write(self.h, f"{' ' * self.li_tag_indent * self.indent}{bullet} ")
-            self.set_text_color()
+            self.set_text_color(*self.font_color)
         if tag == "font":
             # save previous font state:
             self.font_stack.append((self.font_face, self.font_size, self.font_color))
             if "color" in attrs:
-                color = hex2dec(attrs["color"])
+                color = color_as_decimal(attrs["color"])
                 self.font_color = color
             if "face" in attrs:
                 face = attrs.get("face").lower()
@@ -293,7 +459,7 @@ class HTML2FPDF(HTMLParser):
             if "size" in attrs:
                 self.font_size = int(attrs.get("size"))
             self.set_font()
-            self.set_text_color()
+            self.set_text_color(*self.font_color)
         if tag == "table":
             self.table = {k.lower(): v for k, v in attrs.items()}
             if "width" not in self.table:
@@ -391,18 +557,19 @@ class HTML2FPDF(HTMLParser):
     def handle_endtag(self, tag):
         # Closing tag
         LOGGER.debug("ENDTAG %s", tag)
-        if tag in self.hsize:
+        if tag in self.heading_sizes:
             self.heading_level = None
-            self.pdf.ln(self.h)
-            self.set_font()
-            self.set_text_color()
-            self.align = None
-        if tag == "pre":
-            # recover last font state, color is ignored as pre doesn't change it
             face, size, color = self.font_stack.pop()
             self.set_font(face, size)
+            self.set_text_color(*color)
+            self.pdf.ln(self.h)
+            self.align = None
+        if tag == "pre":
+            face, size, color = self.font_stack.pop()
+            self.set_font(face, size)
+            self.set_text_color(*color)
         if tag == "blockquote":
-            self.set_text_color()
+            self.set_text_color(*self.font_color)
             self.indent -= 1
             self.pdf.ln(3)
         if tag == "strong":
@@ -456,7 +623,7 @@ class HTML2FPDF(HTMLParser):
             face, size, color = self.font_stack.pop()
             self.font_color = color
             self.set_font(face, size)
-            self.set_text_color()
+            self.set_text_color(*self.font_color)
         if tag == "center":
             self.align = None
 
@@ -482,10 +649,7 @@ class HTML2FPDF(HTMLParser):
         self.pdf.set_font(style=style)
 
     def set_text_color(self, r=None, g=0, b=0):
-        if r is None:
-            self.pdf.set_text_color(*self.font_color)
-        else:
-            self.pdf.set_text_color(r, g, b)
+        self.pdf.set_text_color(r, g, b)
 
     def put_link(self, txt):
         # Put a hyperlink
@@ -493,13 +657,13 @@ class HTML2FPDF(HTMLParser):
         self.set_style("u", True)
         self.pdf.write(self.h, txt, self.href)
         self.set_style("u", False)
-        self.set_text_color()
+        self.set_text_color(*self.font_color)
 
     # pylint: disable=no-self-use
     def render_toc(self, pdf, outline):
         "This method can be overriden by subclasses to customize the Table of Contents style."
         pdf.ln()
-        for section in outline[1:]:  # skipping top-level h1
+        for section in outline:
             link = pdf.add_link()
             pdf.set_link(link, page=section.page_number)
             text = f'{" " * section.level * 2} {section.name}'
@@ -512,11 +676,13 @@ class HTML2FPDF(HTMLParser):
 
 
 class HTMLMixin:
+    HTML2FPDF_CLASS = HTML2FPDF
+
     def write_html(self, text, *args, **kwargs):
         """Parse HTML and convert it to PDF"""
         kwargs2 = vars(self)
         # Method arguments must override class & instance attributes:
         kwargs2.update(kwargs)
-        h2p = HTML2FPDF(self, *args, **kwargs2)
+        h2p = self.HTML2FPDF_CLASS(self, *args, **kwargs2)
         text = html.unescape(text)  # To deal with HTML entities
         h2p.feed(text)
