@@ -31,15 +31,13 @@ def load_image(filename):
 
 def _decode_base64_image(base64Image):
     "Decode the base 64 image string into an io byte stream."
-
     imageData = base64Image.split("base64,")[1]
     decodedData = base64.b64decode(imageData)
     imageBytes = BytesIO(decodedData)
-
     return imageBytes
 
 
-def get_img_info(img, image_filter="AUTO"):
+def get_img_info(img, image_filter="AUTO", dims=None):
     """
     Args:
         img: `BytesIO` or `PIL.Image.Image` instance
@@ -47,6 +45,8 @@ def get_img_info(img, image_filter="AUTO"):
     """
     if not isinstance(img, Image.Image):
         img = Image.open(img)
+    if dims:
+        img = img.resize(dims, resample=Image.ANTIALIAS)
     if image_filter == "AUTO":
         # Very simple logic for now:
         image_filter = "DCTDecode" if img.format == "JPEG" else "FlateDecode"
@@ -61,7 +61,10 @@ def get_img_info(img, image_filter="AUTO"):
         dpn, bpc, colspace = 1, 8, "DeviceGray"
         alpha_channel = slice(1, None, 2)
         info["data"] = _to_data(img, image_filter, remove_slice=alpha_channel)
-        if image_filter not in ("DCTDecode", "JPXDecode"):
+        if _has_alpha(img, alpha_channel) and image_filter not in (
+            "DCTDecode",
+            "JPXDecode",
+        ):
             info["smask"] = _to_data(img, image_filter, select_slice=alpha_channel)
     elif img.mode == "RGB":
         dpn, bpc, colspace = 3, 8, "DeviceRGB"
@@ -70,7 +73,10 @@ def get_img_info(img, image_filter="AUTO"):
         dpn, bpc, colspace = 3, 8, "DeviceRGB"
         alpha_channel = slice(3, None, 4)
         info["data"] = _to_data(img, image_filter, remove_slice=alpha_channel)
-        if image_filter not in ("DCTDecode", "JPXDecode"):
+        if _has_alpha(img, alpha_channel) and image_filter not in (
+            "DCTDecode",
+            "JPXDecode",
+        ):
             info["smask"] = _to_data(img, image_filter, select_slice=alpha_channel)
 
     dp = f"/Predictor 15 /Colors {dpn} /BitsPerComponent {bpc} /Columns {w}"
@@ -123,3 +129,8 @@ def _to_zdata(img, remove_slice=None, select_slice=None):
         data[i:i] = b"\0"
         i += loop_incr
     return zlib.compress(data)
+
+
+def _has_alpha(img, alpha_channel):
+    alpha = bytearray(img.tobytes())[alpha_channel]
+    return any(c != 255 for c in alpha)
