@@ -54,6 +54,7 @@ class FlexTemplate:
             "L": self._line,
             "I": self._image,
             "B": self._rect,
+            "E": self._ellipse,
             "BC": self._barcode,
             "C39": self._code39,
             "W": self._write,
@@ -115,7 +116,7 @@ class FlexTemplate:
                     raise KeyError(f"Mandatory key '{k}' missing in input data")
             # x2 is optional for barcode types, but needed for offset rendering
             if "x2" not in e:
-                if e["type"] in ["B", "C39"]:
+                if e["type"] in ["BC", "C39"]:
                     e["x2"] = 0
                 else:
                     raise KeyError("Mandatory key 'x2' missing in input data")
@@ -124,8 +125,14 @@ class FlexTemplate:
                     e["size"] = e["w"]
             for k, t in key_config.items():
                 if k in e and not isinstance(e[k], t):
+                    # pylint: disable=no-member
+                    ttype = (
+                        t.__name__
+                        if isinstance(t, type)
+                        else " or ".join([f"'{x.__name__}'" for x in t])
+                    )
                     raise TypeError(
-                        f'Value of element item "{k}" must be {t}, not {type(e[k])}.'
+                        f"Value of element item '{k}' must be {ttype}, not '{type(e[k]).__name__}'."
                     )
             self.keys.append(e["name"].lower())
 
@@ -208,7 +215,7 @@ class FlexTemplate:
                     vs = val.strip()
                     if not vs:
                         if cfg[2]:  # mandatory
-                            if cfg[0] == "x2" and row["type"] in ["B", "C39"]:
+                            if cfg[0] == "x2" and row[1] in ["BC", "C39"]:
                                 # two types don't need x2, but offset rendering does
                                 pass
                             else:
@@ -225,6 +232,10 @@ class FlexTemplate:
         self.keys = [val["name"].lower() for val in self.elements]
 
     def __setitem__(self, name, value):
+        assert isinstance(
+            name, str
+        ), f"name must be of type 'str', not '{type(name).__name__}'."
+        # value has too many valid types to reasonably check here
         if name.lower() not in self.keys:
             raise FPDFException(f"Element not loaded, cannot set item: {name}")
         self.texts[name.lower()] = value
@@ -233,11 +244,17 @@ class FlexTemplate:
     set = __setitem__
 
     def __contains__(self, name):
+        assert isinstance(
+            name, str
+        ), f"name must be of type 'str', not '{type(name).__name__}'."
         return name.lower() in self.keys
 
     def __getitem__(self, name):
+        assert isinstance(
+            name, str
+        ), f"name must be of type 'str', not '{type(name).__name__}'."
         if name not in self.keys:
-            return None
+            raise KeyError(name)
         key = name.lower()
         if key in self.texts:
             # text for this page:
@@ -290,7 +307,6 @@ class FlexTemplate:
 
     def _text(
         self,
-        rotations,
         *_,
         x1=0,
         y1=0,
@@ -305,7 +321,7 @@ class FlexTemplate:
         underline=False,
         align="",
         foreground=0,
-        background=0xFFFFFF,
+        background=None,
         multiline=None,
         **__,
     ):
@@ -314,8 +330,12 @@ class FlexTemplate:
         pdf = self.pdf
         if pdf.text_color != _rgb_as_str(foreground):
             pdf.set_text_color(*_rgb(foreground))
-        if pdf.fill_color != _rgb_as_str(background):
-            pdf.set_fill_color(*_rgb(background))
+        if background is None:
+            fill = False
+        else:
+            fill = True
+            if pdf.fill_color != _rgb_as_str(background):
+                pdf.set_fill_color(*_rgb(background))
 
         font = font.strip().lower()
         style = ""
@@ -330,64 +350,26 @@ class FlexTemplate:
         if underline:
             style += "U"
         pdf.set_font(font, style, size * scale)
+        pdf.set_xy(x1, y1)
         width, height = x2 - x1, y2 - y1
-        rotate = __.get("rotate")
-        if rotate:
-            rotations.append((rotate, x1, y2))
         if multiline is None:  # write without wrapping/trimming (default)
-            self._render_rotated(
-                (x1, y1),
-                pdf.cell,
-                (),
-                {
-                    "w": width,
-                    "h": height,
-                    "txt": text,
-                    "border": 0,
-                    "ln": 0,
-                    "align": align,
-                    "fill": True,
-                },
-                rotations,
+            pdf.cell(
+                w=width, h=height, txt=text, border=0, ln=0, align=align, fill=fill
             )
         elif multiline:  # automatic word - warp
-            self._render_rotated(
-                (x1, y1),
-                pdf.multi_cell,
-                (),
-                {
-                    "w": width,
-                    "h": height,
-                    "txt": text,
-                    "border": 0,
-                    "align": align,
-                    "fill": True,
-                },
-                rotations,
+            pdf.multi_cell(
+                w=width, h=height, txt=text, border=0, align=align, fill=fill
             )
         else:  # trim to fit exactly the space defined
             text = pdf.multi_cell(
                 w=width, h=height, txt=text, align=align, split_only=True
             )[0]
-            self._render_rotated(
-                (x1, y1),
-                pdf.cell,
-                (),
-                {
-                    "w": width,
-                    "h": height,
-                    "txt": text,
-                    "border": 0,
-                    "ln": 0,
-                    "align": align,
-                    "fill": True,
-                },
-                rotations,
+            pdf.cell(
+                w=width, h=height, txt=text, border=0, ln=0, align=align, fill=fill
             )
 
     def _line(
         self,
-        rotations,
         *_,
         x1=0,
         y1=0,
@@ -398,18 +380,13 @@ class FlexTemplate:
         foreground=0,
         **__,
     ):
-        pdf = self.pdf
-        if pdf.draw_color.lower() != _rgb_as_str(foreground):
-            pdf.set_draw_color(*_rgb(foreground))
-        pdf.set_line_width(size * scale)
-        rotate = __.get("rotate")
-        if rotate:
-            rotations.append((rotate, x1, y2))
-        self._render_rotated(None, pdf.line, (x1, y1, x2, y2), {}, rotations)
+        if self.pdf.draw_color.lower() != _rgb_as_str(foreground):
+            self.pdf.set_draw_color(*_rgb(foreground))
+        self.pdf.set_line_width(size * scale)
+        self.pdf.line(x1, y1, x2, y2)
 
     def _rect(
         self,
-        rotations,
         *_,
         x1=0,
         y1=0,
@@ -418,38 +395,52 @@ class FlexTemplate:
         size=0,
         scale=1.0,
         foreground=0,
-        background=0xFFFFFF,
+        background=None,
         **__,
     ):
         pdf = self.pdf
         if pdf.draw_color.lower() != _rgb_as_str(foreground):
             pdf.set_draw_color(*_rgb(foreground))
-        if pdf.fill_color != _rgb_as_str(background):
-            pdf.set_fill_color(*_rgb(background))
+        if background is None:
+            style = "D"
+        else:
+            style = "FD"
+            if pdf.fill_color != _rgb_as_str(background):
+                pdf.set_fill_color(*_rgb(background))
         pdf.set_line_width(size * scale)
-        rotate = __.get("rotate")
-        if rotate:
-            rotations.append((rotate, x1, y2))
-        self._render_rotated(
-            None, pdf.rect, (x1, y1, x2 - x1, y2 - y1), {"style": "FD"}, rotations
-        )
+        pdf.rect(x1, y1, x2 - x1, y2 - y1, style=style)
 
-    def _image(self, rotations, *_, x1=0, y1=0, x2=0, y2=0, text="", **__):
+    def _ellipse(
+        self,
+        *_,
+        x1=0,
+        y1=0,
+        x2=0,
+        y2=0,
+        size=0,
+        scale=1.0,
+        foreground=0,
+        background=None,
+        **__,
+    ):
+        pdf = self.pdf
+        if pdf.draw_color.lower() != _rgb_as_str(foreground):
+            pdf.set_draw_color(*_rgb(foreground))
+        if background is None:
+            style = "D"
+        else:
+            style = "FD"
+            if pdf.fill_color != _rgb_as_str(background):
+                pdf.set_fill_color(*_rgb(background))
+        pdf.set_line_width(size * scale)
+        pdf.ellipse(x1, y1, x2 - x1, y2 - y1, style=style)
+
+    def _image(self, *_, x1=0, y1=0, x2=0, y2=0, text="", **__):
         if text:
-            rotate = __.get("rotate")
-            if rotate:
-                rotations.append((rotate, x1, y2))
-            self._render_rotated(
-                None,
-                self.pdf.image,
-                (text, x1, y1),
-                {"w": x2 - x1, "h": y2 - y1, "link": ""},
-                rotations,
-            )
+            self.pdf.image(text, x1, y1, w=x2 - x1, h=y2 - y1, link="")
 
     def _barcode(
         self,
-        rotations,
         *_,
         x1=0,
         y1=0,
@@ -468,20 +459,10 @@ class FlexTemplate:
             pdf.set_fill_color(*_rgb(foreground))
         font = font.lower().strip()
         if font == "interleaved 2of5 nt":
-            rotate = __.get("rotate")
-            if rotate:
-                rotations.append((rotate, x1, y2))
-            self._render_rotated(
-                None,
-                pdf.interleaved2of5,
-                (text, x1, y1),
-                {"w": size * scale, "h": y2 - y1},
-                rotations,
-            )
+            pdf.interleaved2of5(text, x1, y1, w=size * scale, h=y2 - y1)
 
     def _code39(
         self,
-        rotations,
         *_,
         x1=0,
         y1=0,
@@ -507,18 +488,12 @@ class FlexTemplate:
         h = y2 - y1
         if h <= 0:
             h = 5
-        rotate = __.get("rotate")
-        if rotate:
-            rotations.append((rotate, x1, y2))
-        self._render_rotated(
-            None, pdf.code39, (text, x1, y1, size * scale, h), {}, rotations
-        )
+        pdf.code39(text, x1, y1, size * scale, h)
 
     # Added by Derek Schwalenberg Schwalenberg1013@gmail.com to allow (url) links in
     # templates (using write method) 2014-02-22
     def _write(
         self,
-        rotations,
         *_,
         x1=0,
         y1=0,
@@ -554,26 +529,8 @@ class FlexTemplate:
         if underline:
             style += "U"
         pdf.set_font(font, style, size * scale)
-        rotate = __.get("rotate")
-        if rotate:
-            rotations.append((rotate, x1, y2))
-        self._render_rotated((x1, y1), pdf.write, (5, text, link), {}, rotations)
-
-    def _render_rotated(self, pos, func, args, kwargs, rotations):
-        # Solves issue 226
-        # Settings operations (fonts, colors, line widths etc.) must not appear
-        # within a rotation context.
-        # The solution is to queue up rotations and execute them all in one go
-        # once everything else has been set up.
-        # Technically, we could keep rotating until we're dizzy (up to Pythons
-        # recursion limit), but in practise we take two turns at most.
-        if rotations:
-            with self.pdf.rotation(*(rotations[0])):
-                self._render_rotated(pos, func, args, kwargs, rotations[1:])
-        else:
-            if pos:
-                self.pdf.set_xy(*pos)
-            func(*args, **kwargs)
+        pdf.set_xy(x1, y1)
+        pdf.write(5, text, link)
 
     def render(self, offsetx=0.0, offsety=0.0, rotate=0.0, scale=1.0):
         """
@@ -608,12 +565,18 @@ class FlexTemplate:
             ele["scale"] = scale
             handler_name = ele["type"].upper()
             if rotate:  # don't rotate by 0.0 degrees
-                rotations = [
-                    (rotate, offsetx, offsety),
-                ]
-                self.handlers[handler_name](rotations, **ele)
+                with self.pdf.rotation(rotate, offsetx, offsety):
+                    if "rotate" in ele and ele["rotate"]:
+                        with self.pdf.rotation(ele["rotate"], ele["x1"], ele["y1"]):
+                            self.handlers[handler_name](**ele)
+                    else:
+                        self.handlers[handler_name](**ele)
             else:
-                self.handlers[handler_name]([], **ele)
+                if "rotate" in ele and ele["rotate"]:
+                    with self.pdf.rotation(ele["rotate"], ele["x1"], ele["y1"]):
+                        self.handlers[handler_name](**ele)
+                else:
+                    self.handlers[handler_name](**ele)
         self.texts = {}  # reset modified entries for the next page
 
 
